import { getVersionName, getVersionCode } from '@nativescript/appversion';
import { Http, ApplicationSettings, Utils, Device } from '@nativescript/core';
import * as moment from 'moment/moment';
export class Common {
    constructor() {
        this.maxAmountOfBreadcrumbs = 50;
        this.storageKey = 'sentryCrashesQueue';
        this.breadcrumbs = [];
        this.versionName = '';
        this.versionCode = '';
        this.sentryId = '';
        this.environment = 'DEVELOPMENT';
        this.userId = 'unknown';
        if (Common.getInstance()) {
            throw new Error("Error: Instance failed: Use LiveEngage.getInstance() instead of new.");
        }
        Common.instance = this;
        setTimeout(() => {
            this.setVersionName();
            this.setVersionCode();
        });
    }
    static getInstance() {
        return Common.instance;
    }
    setCredentials(sentryId, sentryKey, environment, userId) {
        this.sentryId = sentryId;
        this.endPoint = 'https://sentry.io/api/' + sentryId + '/store/?sentry_version=7&sentry_client=raven-js%2F3.22.2&sentry_key=' + sentryKey;
        if (environment) {
            this.environment = environment;
        }
        if (userId) {
            this.userId = userId;
        }
    }
    setMaxAmountOfBreadcrumbs(amount) {
        this.maxAmountOfBreadcrumbs = amount;
    }
    saveBreadcrumb(message, category, properties) {
        const crumb = {
            timestamp: moment(),
            message,
            category,
            data: properties
        };
        this.breadcrumbs.push(crumb);
        if (this.breadcrumbs.length > this.maxAmountOfBreadcrumbs) {
            this.breadcrumbs.shift();
        }
    }
    saveCrash(errorMessage, errorDetails) {
        this.saveBreadcrumb('' + errorDetails, 'crash start');
        let deviceType = '';
        let language = '';
        let manufacturer = '';
        let model = '';
        let os = '';
        let osVersion = '';
        if (!Utils.isNullOrUndefined(Device)) {
            deviceType = Device.deviceType;
            language = Device.language;
            manufacturer = Device.manufacturer;
            model = Device.model;
            os = Device.os;
            osVersion = Device.osVersion;
        }
        const crashData = {
            project: this.sentryId,
            logger: 'nativescript',
            platform: 'javascript',
            message: '' + errorMessage,
            level: 'error',
            tags: {
                device_type: deviceType,
                device_lang: language,
                app_version: this.versionCode,
                trace_category: 'Error'
            },
            breadcrumbs: {
                values: this.breadcrumbs
            },
            environment: this.environment,
            release: this.versionName,
            contexts: {
                device: {
                    family: manufacturer,
                    model: model
                },
                os: {
                    name: os,
                    version: osVersion
                },
                runtime: {
                    name: 'Nativescript'
                }
            },
            user: {
                id: this.userId,
            }
        };
        let crashes = [];
        if (this.isPresentInSettings(this.storageKey)) {
            crashes = JSON.parse(ApplicationSettings.getString(this.storageKey));
        }
        crashes.push(crashData);
        ApplicationSettings.setString(this.storageKey, JSON.stringify(crashes));
    }
    sendCrashes() {
        if (!this.isPresentInSettings(this.storageKey)) {
            return;
        }
        const crashes = JSON.parse(ApplicationSettings.getString(this.storageKey));
        for (let crash of crashes) {
            this.submitCrash(crash);
        }
    }
    submitCrash(crash) {
        Http.request({
            url: this.endPoint,
            method: "POST",
            headers: {
                "Content-Type": "application/json",
                'Accept': '*/*',
                'Origin': 'nativescript://'
            },
            content: JSON.stringify(crash)
        }).then((response) => {
            const crashes = JSON.parse(ApplicationSettings.getString(this.storageKey));
            const index = this.customIndexOf(crashes, crash);
            if (index > -1) {
                crashes.splice(index, 1);
                ApplicationSettings.setString(this.storageKey, JSON.stringify(crashes));
            }
        }, (e) => {
            console.error("Sentry send error", e);
        });
    }
    setVersionName() {
        getVersionName().then((versionName) => {
            this.versionName = versionName;
        });
    }
    setVersionCode() {
        getVersionCode().then((versionCode) => {
            this.versionCode = versionCode;
        });
    }
    isPresentInSettings(storeKey) {
        return ApplicationSettings.getString(storeKey) !== undefined;
    }
    customIndexOf(arr, searchElement) {
        for (let i = 0; i < arr.length; i++) {
            if (arr[i].message === searchElement.message) {
                return i;
            }
        }
        return -1;
    }
}
Common.instance = new Common();
//# sourceMappingURL=ng-sentry.common.js.map